searchNodes=[{"doc":"The PieceTable module provides a naive implementation of the piece-table data structure for efficient text editing operations. A piece-table represents an editable buffer of text as a sequence of non-overlapping pieces, allowing efficient inserts, deletes, and modifications. Usage iex&gt; table = PieceTable . new! ( &quot;Hello, world!&quot; ) iex&gt; table = PieceTable . insert! ( table , &quot;you &quot; , 7 ) iex&gt; table = PieceTable . delete! ( table , 10 , 6 ) iex&gt; table = PieceTable . undo! ( table ) iex&gt; table = PieceTable . redo! ( table ) iex&gt; PieceTable . get_text! ( table ) &quot;Hello, you!&quot;","ref":"PieceTable.html","title":"PieceTable","type":"module"},{"doc":"Deletes a substring from the PieceTable starting at a specified position and with a specified length. Parameters table (PieceTable.t()): The PieceTable from which the substring will be deleted. position (integer()): The starting position of the substring to be deleted. The position is zero-based. length (integer()): The length of the substring to be deleted. Returns {:ok, PieceTable.t()} {:error, &quot;invalid arguments&quot;} Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; PieceTable . delete ( table , 4 , 3 ) { :ok , % PieceTable { original : &quot;Initial content&quot; , result : &quot;Init content&quot; , applied : [ % PieceTable.Change { change : :del , text : &quot;ial&quot; , position : 4 } ] } }","ref":"PieceTable.html#delete/3","title":"PieceTable.delete/3","type":"function"},{"doc":"Deletes a substring from the PieceTable starting at a specified position and with a specified length. Parameters table (PieceTable.t()): The PieceTable from which the substring will be deleted. position (integer()): The starting position of the substring to be deleted. The position is zero-based. length (integer()): The length of the substring to be deleted. Returns PieceTable.t() Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; PieceTable . delete! ( table , 4 , 3 ) % PieceTable { original : &quot;Initial content&quot; , result : &quot;Init content&quot; , applied : [ % PieceTable.Change { change : :del , text : &quot;ial&quot; , position : 4 } ] }","ref":"PieceTable.html#delete!/3","title":"PieceTable.delete!/3","type":"function"},{"doc":"Retrieves the entire text content from the PieceTable. Parameters table (PieceTable.t()): The PieceTable from which the text will be retrieved. Returns {:ok, String.t()} {:error, &quot;not a PieceTable struct&quot;} Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; table = PieceTable . delete! ( table , 4 , 3 ) iex&gt; PieceTable . get_text ( table ) { :ok , &quot;Init content&quot; }","ref":"PieceTable.html#get_text/1","title":"PieceTable.get_text/1","type":"function"},{"doc":"Retrieves the entire text content from the PieceTable. Parameters table (PieceTable.t()): The PieceTable from which the text will be retrieved. Returns String.t() Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; table = PieceTable . delete! ( table , 4 , 3 ) iex&gt; PieceTable . get_text! ( table ) &quot;Init content&quot;","ref":"PieceTable.html#get_text!/1","title":"PieceTable.get_text!/1","type":"function"},{"doc":"Inserts text into the PieceTable at a specified position. Parameters table (PieceTable.t()): The PieceTable where the text will be inserted. text (String.t()): The text to be inserted. position (integer()): The position where the text should be inserted. The position is zero-based. Returns {:ok, PieceTable.t()} {:error, &quot;invalid arguments&quot;} Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; PieceTable . insert ( table , &quot;, before&quot; , 15 ) { :ok , % PieceTable { original : &quot;Initial content&quot; , result : &quot;Initial content, before&quot; , applied : [ % PieceTable.Change { change : :ins , text : &quot;, before&quot; , position : 15 } ] } }","ref":"PieceTable.html#insert/3","title":"PieceTable.insert/3","type":"function"},{"doc":"Inserts text into the PieceTable at a specified position. Parameters table (PieceTable.t()): The PieceTable where the text will be inserted. text (String.t()): The text to be inserted. position (integer()): The position where the text should be inserted. The position is zero-based. Returns PieceTable.t() Examples iex&gt; table = PieceTable . new! ( &quot;Initial content&quot; ) iex&gt; PieceTable . insert! ( table , &quot;, before&quot; , 15 ) % PieceTable { original : &quot;Initial content&quot; , result : &quot;Initial content, before&quot; , applied : [ % PieceTable.Change { change : :ins , text : &quot;, before&quot; , position : 15 } ] }","ref":"PieceTable.html#insert!/3","title":"PieceTable.insert!/3","type":"function"},{"doc":"Creates a new PieceTable struct. This is intended the only method to build it. Parameters text (String.t()): The initial content of the piece table. Returns {:ok, %PieceTable{}} {:error, &quot;original text is not a string&quot;} Examples iex&gt; PieceTable . new ( &quot;test&quot; ) { :ok , % PieceTable { original : &quot;test&quot; , result : &quot;test&quot; , applied : [ ] } }","ref":"PieceTable.html#new/1","title":"PieceTable.new/1","type":"function"},{"doc":"Creates a new PieceTable struct. This is intended the only method to build it. Parameters text (String.t()): The initial content of the piece table. Returns %PieceTable{} Examples iex&gt; PieceTable . new! ( &quot;test&quot; ) % PieceTable { original : &quot;test&quot; , result : &quot;test&quot; , applied : [ ] }","ref":"PieceTable.html#new!/1","title":"PieceTable.new!/1","type":"function"},{"doc":"Redo the next change previously undone to the string. Parameters table (PieceTable.t()): The PieceTable from which the text will be retrieved. Returns {:ok, PieceTable.t()} {:last, PieceTable.t()} {:error, &quot;not a PieceTable struct&quot;} Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; table = PieceTable . delete! ( table , 4 , 3 ) iex&gt; { :ok , table } = PieceTable . undo ( table ) iex&gt; PieceTable . redo ( table ) { :ok , % PieceTable { original : &quot;Initial content&quot; , result : &quot;Init content&quot; , applied : [ % PieceTable.Change { change : :del , text : &quot;ial&quot; , position : 4 } ] } }","ref":"PieceTable.html#redo/1","title":"PieceTable.redo/1","type":"function"},{"doc":"Redo the next change previously undone to the string. Parameters table (PieceTable.t()): The PieceTable from which the text will be retrieved. Returns PieceTable.t() Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; table = PieceTable . delete! ( table , 4 , 3 ) iex&gt; { :ok , table } = PieceTable . undo ( table ) iex&gt; PieceTable . redo! ( table ) % PieceTable { original : &quot;Initial content&quot; , result : &quot;Init content&quot; , applied : [ % PieceTable.Change { change : :del , text : &quot;ial&quot; , position : 4 } ] }","ref":"PieceTable.html#redo!/1","title":"PieceTable.redo!/1","type":"function"},{"doc":"Undo the latest change applied to the string. Parameters table (PieceTable.t()): The PieceTable from which the text will be retrieved. Returns {:ok, PieceTable.t()} {:first, PieceTable.t()} {:error, &quot;not a PieceTable struct&quot;} Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; table = PieceTable . delete! ( table , 4 , 3 ) iex&gt; PieceTable . undo ( table ) { :ok , % PieceTable { original : &quot;Initial content&quot; , result : &quot;Initial content&quot; , applied : [ ] , to_apply : [ % PieceTable.Change { change : :del , text : &quot;ial&quot; , position : 4 } ] } }","ref":"PieceTable.html#undo/1","title":"PieceTable.undo/1","type":"function"},{"doc":"Undo the latest change applied to the string. Parameters table (PieceTable.t()): The PieceTable from which the text will be retrieved. Returns PieceTable.t() Examples iex&gt; { :ok , table } = PieceTable . new ( &quot;Initial content&quot; ) iex&gt; table = PieceTable . delete! ( table , 4 , 3 ) iex&gt; PieceTable . undo! ( table ) % PieceTable { original : &quot;Initial content&quot; , result : &quot;Initial content&quot; , applied : [ ] , to_apply : [ % PieceTable.Change { change : :del , text : &quot;ial&quot; , position : 4 } ] }","ref":"PieceTable.html#undo!/1","title":"PieceTable.undo!/1","type":"function"},{"doc":"","ref":"PieceTable.html#update_piece_table/2","title":"PieceTable.update_piece_table/2","type":"function"},{"doc":"","ref":"PieceTable.html#t:t/0","title":"PieceTable.t/0","type":"type"},{"doc":"A struct to hold changes in the PieceTable Usage iex&gt; table = PieceTable.Change . new! ( :ins , &quot;test&quot; , 3 ) % PieceTable.Change { change : :ins , position : 3 , text : &quot;test&quot; }","ref":"PieceTable.Change.html","title":"PieceTable.Change","type":"module"},{"doc":"Inverts the operation of a PieceTable.Change struct. Parameters chg (PieceTable.Change.t()): The change to invert Returns {:ok, %PieceTable.Change{}} {:error, any} Examples iex&gt; PieceTable.Change . invert ( % PieceTable.Change { change : :ins , text : &quot;random&quot; , position : 5 } ) { :ok , % PieceTable.Change { change : :del , text : &quot;random&quot; , position : 5 } }","ref":"PieceTable.Change.html#invert/1","title":"PieceTable.Change.invert/1","type":"function"},{"doc":"Inverts the operation of a PieceTable.Change struct. Parameters chg (PieceTable.Change.t()): The change to invert Returns %PieceTable.Change{} Examples iex&gt; PieceTable.Change . invert ( % PieceTable.Change { change : :ins , text : &quot;random&quot; , position : 5 } ) { :ok , % PieceTable.Change { change : :del , text : &quot;random&quot; , position : 5 } }","ref":"PieceTable.Change.html#invert!/1","title":"PieceTable.Change.invert!/1","type":"function"},{"doc":"Creates a new PieceTable.Change struct. This is intended the only method to build it. Parameters change (atom()): The operation it represents [:ins | :del] text (String.t()): The text to edit position (integer()): The position at which the edit occurs Returns {:ok, %PieceTable.Change{}} {:error, {:wrong, &quot;edit&quot;, -1}} Examples iex&gt; PieceTable.Change . new ( :ins , &quot;test&quot; , 4 ) { :ok , % PieceTable.Change { change : :ins , text : &quot;test&quot; , position : 4 } }","ref":"PieceTable.Change.html#new/3","title":"PieceTable.Change.new/3","type":"function"},{"doc":"Creates a new PieceTable.Change struct. This is intended the only method to build it. Parameters change (atom()): The operation it represents [:ins | :del] text (String.t()): The text to edit position (integer()): The position at which the edit occurs Returns %PieceTable.Change{} Examples iex&gt; PieceTable.Change . new! ( :ins , &quot;test&quot; , 4 ) % PieceTable.Change { change : :ins , text : &quot;test&quot; , position : 4 }","ref":"PieceTable.Change.html#new!/3","title":"PieceTable.Change.new!/3","type":"function"},{"doc":"","ref":"PieceTable.Change.html#t:t/0","title":"PieceTable.Change.t/0","type":"type"},{"doc":"A module to calculate the difference between a text and another text or a piece table. Usage iex&gt; PieceTable.Differ . diff ( &quot;test&quot; , &quot;text&quot; ) { :ok , % PieceTable { original : &quot;test&quot; , result : &quot;text&quot; , applied : [ % PieceTable.Change { change : :ins , text : &quot;x&quot; , position : 2 } , % PieceTable.Change { change : :del , text : &quot;s&quot; , position : 2 } ] , to_apply : [ ] } }","ref":"PieceTable.Differ.html","title":"PieceTable.Differ","type":"module"},{"doc":"Computes the difference between the original input and a modified string using the PieceTable data structure. Parameters original (original_input()): The original input string. modified (String.t()): The modified string. Returns {:ok, %PieceTable{}} : A tuple containing the modified PieceTable structure. {:error, String.t()} : An error message indicating wrong arguments. Examples iex&gt; PieceTable.Differ . diff ( &quot;test&quot; , &quot;text&quot; ) { :ok , % PieceTable { original : &quot;test&quot; , result : &quot;text&quot; , applied : [ % PieceTable.Change { change : :ins , text : &quot;x&quot; , position : 2 } , % PieceTable.Change { change : :del , text : &quot;s&quot; , position : 2 } ] , to_apply : [ ] } } iex&gt; PieceTable.Differ . diff ( &quot;test&quot; , 42 ) { :error , &quot;Wrong arguments&quot; }","ref":"PieceTable.Differ.html#diff/2","title":"PieceTable.Differ.diff/2","type":"function"},{"doc":"Computes the difference between the original input and a modified string using the PieceTable data structure. Parameters original (original_input()): The original input string. modified (String.t()): The modified string. Returns %PieceTable{} : A tuple containing the modified PieceTable structure. Examples iex&gt; PieceTable.Differ . diff! ( &quot;test&quot; , &quot;text&quot; ) % PieceTable { original : &quot;test&quot; , result : &quot;text&quot; , applied : [ % PieceTable.Change { change : :ins , text : &quot;x&quot; , position : 2 } , % PieceTable.Change { change : :del , text : &quot;s&quot; , position : 2 } ] , to_apply : [ ] }","ref":"PieceTable.Differ.html#diff!/2","title":"PieceTable.Differ.diff!/2","type":"function"},{"doc":"","ref":"PieceTable.Differ.html#t:original_input/0","title":"PieceTable.Differ.original_input/0","type":"type"}]